package com.ruoyi.web.controller.ss;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.ss.vip.domain.Vip;
import com.ruoyi.ss.vip.domain.VipVO;
import com.ruoyi.ss.vip.domain.VipQuery;
import com.ruoyi.ss.vip.service.VipService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 会员Controller
 *
 * @author ruoyi
 * @date 2024-12-09
 */
@RestController
@RequestMapping("/ss/vip")
public class VipController extends BaseController
{
    @Autowired
    private VipService vipService;

    /**
     * 查询会员列表
     */
    @PreAuthorize("@ss.hasPermi('ss:vip:list')")
    @GetMapping("/list")
    public TableDataInfo list(VipQuery query)
    {
        startPage();
        startOrderBy();
        List<VipVO> list = vipService.selectVipList(query);
        return getDataTable(list);
    }

    /**
     * 导出会员列表
     */
    @PreAuthorize("@ss.hasPermi('ss:vip:export')")
    @Log(title = "会员", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, VipQuery query)
    {
        List<VipVO> list = vipService.selectVipList(query);
        ExcelUtil<VipVO> util = new ExcelUtil<VipVO>(VipVO.class);
        util.exportExcel(response, list, "会员数据");
    }

    /**
     * 获取会员详细信息
     */
    @PreAuthorize("@ss.hasPermi('ss:vip:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(vipService.selectVipById(id));
    }

    /**
     * 新增会员
     */
    @PreAuthorize("@ss.hasPermi('ss:vip:add')")
    @Log(title = "会员", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody Vip vip)
    {
        return toAjax(vipService.insertVip(vip));
    }

    /**
     * 修改会员
     */
    @PreAuthorize("@ss.hasPermi('ss:vip:edit')")
    @Log(title = "会员", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody Vip vip) {
        return toAjax(vipService.updateVip(vip));
    }

    /**
     * 删除会员
     */
    @PreAuthorize("@ss.hasPermi('ss:vip:remove')")
    @Log(title = "会员", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(vipService.deleteVipByIds(ids));
    }
}
